/*
 * Copyright (C) 2009-2011 the libgit2 contributors
 *
 * This file is part of libgit2, distributed under the GNU GPL v2 with
 * a Linking Exception. For full terms see the included COPYING file.
 */

#ifndef INCLUDE_git_git_h__
#define INCLUDE_git_git_h__

#include "git2/version.h"

#include "git2/common.h"
#include "git2/threads.h"
#include "git2/errors.h"
#include "git2/zlib.h"

#include "git2/types.h"

#include "git2/oid.h"
#include "git2/signature.h"
#include "git2/odb.h"

#include "git2/repository.h"
#include "git2/revwalk.h"
#include "git2/refs.h"
#include "git2/reflog.h"

#include "git2/object.h"
#include "git2/blob.h"
#include "git2/commit.h"
#include "git2/tag.h"
#include "git2/tree.h"

#include "git2/index.h"
#include "git2/config.h"
#include "git2/remote.h"

#include "git2/refspec.h"
#include "git2/net.h"
#include "git2/status.h"
#include "git2/indexer.h"

#endif
